#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for Prismic library"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

import prismic
from prismic import connection
import unittest

# logging.basicConfig(level=logging.DEBUG)
# log = logging.getLogger(__name__)


class ConnectionTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        """Teardown."""

    def test_missing_header_key(self):
        headers = {}
        max_age = connection.get_max_age(headers)
        self.assertEqual(max_age, None)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
prismic.connection
~~~~~~~~~~~

This module implements the Prismic Connection handlers.

"""

try:  # 2.7
    import urllib.parse as urlparse
except ImportError:  # 3.x
    import urllib as urlparse

import requests
import json
import re
import platform
from collections import OrderedDict
from requests.exceptions import InvalidSchema
from .exceptions import (InvalidTokenError, AuthorizationNeededError,
                         HTTPError, InvalidURLError)
from .cache import ShelveCache
from . import __version__ as prismic_version

def get_using_requests(full_url):
    request = requests.get(full_url, headers={
        "Accept": "application/json",
        "User-Agent": "Prismic-python-kit/%s Python/%s" % (
            prismic_version,
            platform.python_version()
        )
    })
    return request.status_code, request.text, request.headers

def get_json(url, params=None, access_token=None, cache=None, ttl=None, request_handler=None):
    full_params = dict() if params is None else params.copy()
    if cache is None:
        cache = ShelveCache(re.sub(r'/\\', '', url.split('/')[2]))
    if request_handler is None:
        request_handler = get_using_requests
    if access_token is not None:
        full_params["access_token"] = access_token
    full_url = url if len(full_params) == 0 else (url + "?" + urlparse.urlencode(full_params, doseq=1))
    cached = cache.get(full_url)
    if cached is not None:
        return cached
    try:
        status_code, text_result, headers = request_handler(full_url)
        if status_code == 200:
            json_result = json.loads(text_result, object_pairs_hook=OrderedDict)
            expire = ttl or get_max_age(headers)
            if expire is not None:
                cache.set(full_url, json_result, expire)
            return json_result
        elif status_code == 401:
            if len(access_token) == 0:
                raise AuthorizationNeededError()
            else:
                raise InvalidTokenError()
        else:
            raise HTTPError(status_code, str(text_result))
    except InvalidSchema as e:
        raise InvalidURLError(e)


def get_max_age(headers):
    expire_header = headers.get("Cache-Control", None)
    if expire_header is not None:
        m = re.match("max-age=(\d+)", expire_header)
        if m:
            return int(m.group(1))
    return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for Prismic library"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

from prismic.cache import ShelveCache
from prismic.exceptions import InvalidTokenError, AuthorizationNeededError, InvalidURLError
from .test_prismic_fixtures import fixture_api, fixture_search, fixture_groups, \
    fixture_structured_lists, fixture_empty_paragraph, fixture_store_geopoint, fixture_image_links, \
    fixture_spans_labels, fixture_block_labels, fixture_custom_html, fixture_slices, fixture_composite_slices
import time
import json
import logging
import prismic
from prismic import predicates
import unittest

# logging.basicConfig(level=logging.DEBUG)
# log = logging.getLogger(__name__)


class PrismicTestCase(unittest.TestCase):
    def setUp(self):
        """Init the api url and the token identifier."""
        self.api_url = "http://micro.prismic.io/api"
        self.token = "MC5VcXBHWHdFQUFONDZrbWp4.77-9cDx6C3lgJu-_vXZafO-_vXPvv73vv73vv70777-9Ju-_ve-_vSLvv73vv73vv73vv70O77-977-9Me-_vQ"
        self.fixture_api = json.loads(fixture_api)
        self.fixture_search = json.loads(fixture_search)
        self.fixture_structured_lists = json.loads(fixture_structured_lists)
        self.fixture_empty_paragraph = json.loads(fixture_empty_paragraph)
        self.fixture_block_labels = json.loads(fixture_block_labels)
        self.fixture_store_geopoint = json.loads(fixture_store_geopoint)
        self.fixture_groups = json.loads(fixture_groups)
        self.fixture_image_links = json.loads(fixture_image_links)
        self.fixture_spans_labels = json.loads(fixture_spans_labels)
        self.fixture_custom_html = json.loads(fixture_custom_html)
        self.fixture_slices = json.loads(fixture_slices)
        self.fixture_composite_slices = json.loads(fixture_composite_slices)

        self.api = prismic.Api(self.fixture_api, self.token, ShelveCache("prismictest"), None)

    def tearDown(self):
        """Teardown."""

    @staticmethod
    def link_resolver(document_link):
        if document_link.is_broken:
            return "#broken"
        else:
            return "/document/%s/%s" % (document_link.id, document_link.slug)

    @staticmethod
    def html_serializer(element, content):
        if isinstance(element, prismic.fragments.Block.Image):
            return element.get_view().as_html(PrismicTestCase.link_resolver)
        if isinstance(element, prismic.fragments.Span.Hyperlink):
            return """<a class="some-link" href="%s">""" % element.get_url(PrismicTestCase.link_resolver) + content + "</a>"
        return None


class ApiIntegrationTestCase(PrismicTestCase):
    """Doing real HTTP requests to test API data fetching"""

    def setUp(self):
        super(ApiIntegrationTestCase, self).setUp()
        self.api = prismic.get(self.api_url, self.token)

    def test_get_api(self):
        self.assertGreater(len(self.api.forms), 0)

    def test_api_get_errors(self):
        with self.assertRaises(InvalidTokenError):
            prismic.get(self.api_url, "wrong")

        with self.assertRaises(AuthorizationNeededError):
            prismic.get(self.api_url, "")

        with self.assertRaises(InvalidURLError):
            prismic.get("htt://wrong_on_purpose", "")

    def test_search_form(self):
        form = self.api.form("everything")
        form.ref(self.api.get_master())
        docs = form.submit().documents
        self.assertGreaterEqual(len(docs), 2)

    def test_search_form_orderings(self):
        form = self.api.form("everything")
        form.ref(self.api.get_master())
        form.query('[[:q = at(document.type, "all")]]')
        form.orderings("[my.all.number]")
        docs = form.submit().documents
        self.assertEqual(docs[0].uid, 'all')
        self.assertEqual(docs[1].uid, 'all1')
        self.assertEqual(docs[2].uid, 'all2')

    def test_search_form_page_size(self):
        form = self.api.form("everything").page_size(2)
        form.ref(self.api.get_master())
        response = form.submit()
        self.assertEqual(len(response.documents), 2)
        self.assertEqual(response.results_per_page, 2)

    def test_search_form_first_page(self):
        form = self.api.form("everything").pageSize(2)
        form.ref(self.api.get_master())
        response = form.submit()
        self.assertEqual(response.page, 1)
        self.assertEqual(len(response.documents), 2)
        self.assertEqual(response.results_size, len(response.documents))
        self.assertIsNone(response.prev_page)
        self.assertIsNotNone(response.next_page)

    def test_search_form_page(self):
        form = self.api.form("everything").pageSize(2).page(2)
        form.ref(self.api.get_master())
        response = form.submit()
        self.assertEqual(response.page, 2)
        self.assertEqual(len(response.documents), 2)
        self.assertEqual(response.results_size, len(response.documents))
        self.assertIsNotNone(response.prev_page)
        self.assertIsNotNone(response.next_page)

    def test_search_form_count(self):
        form = self.api.form("everything")
        form.ref(self.api.get_master())
        nb_docs = form.count()
        self.assertGreaterEqual(nb_docs, 2)

    def test_query(self):
        doc = self.api\
            .query(predicates.at('document.id', 'WHx-gSYAAMkyXYX_'))\
            .documents[0]
        self.assertEqual(doc.id, 'WHx-gSYAAMkyXYX_')

    def test_query_first(self):
        doc = self.api.query_first(predicates.at('document.id', 'WHx-gSYAAMkyXYX_'))
        self.assertEqual(doc.id, 'WHx-gSYAAMkyXYX_')

    def test_query_first_no_result(self):
        doc = self.api.query_first(predicates.at('document.id', 'NotAValidId'))
        self.assertIsNone(doc)

    def test_get_by_uid(self):
        doc = self.api.get_by_uid('all', 'all')
        self.assertEqual(doc.id, 'WHx-gSYAAMkyXYX_')

    def test_get_by_id(self):
        doc = self.api.get_by_id('WHx-gSYAAMkyXYX_')
        self.assertEqual(doc.id, 'WHx-gSYAAMkyXYX_')

    def test_get_by_ids(self):
        result = self.api.get_by_ids(['WHx-gSYAAMkyXYX_', 'WHyJqyYAAHgyXbcj'])
        ids = sorted([doc.id for doc in result.documents])
        self.assertEqual(ids[0], 'WHx-gSYAAMkyXYX_')
        self.assertEqual(ids[1], 'WHyJqyYAAHgyXbcj')

    def test_get_single(self):
        doc = self.api.get_single('single')
        self.assertEqual(doc.id, 'V_OplCUAACQAE0lA')

    def test_linked_documents(self):
        doc = self.api\
            .form("everything")\
            .ref(self.api.get_master())\
            .query('[[:d = at(document.id, "WHx-gSYAAMkyXYX_")]]')\
            .submit()\
            .documents[0]
        self.assertEqual(len(doc.linked_documents), 2)

    def test_fetch_links(self):
        article = self.api\
            .form('everything')\
            .ref(self.api.get_master())\
            .fetch_links('all.text')\
            .query(predicates.at('document.id', 'WHx-gSYAAMkyXYX_')) \
            .submit()\
            .documents[0]
        links = article.get_all('all.link_document')
        self.assertEqual(links[0].get_text('all.text'), 'all1')

    def test_fetch_links_list(self):
        article = self.api\
            .form('everything')\
            .ref(self.api.get_master())\
            .fetch_links(['all.text', 'all.number'])\
            .query(predicates.at('document.id', 'WH2PaioAALYBEgug')) \
            .submit()\
            .documents[0]
        links = article.get_all('all.link_document')
        self.assertEqual(links[0].get_text('all.text'), 'all')
        self.assertEqual(links[0].get_text('all.number'), 20)


class ApiTestCase(PrismicTestCase):
    def test_get_ref_master(self):
        self.assertEqual(self.api.get_ref("Master").ref, "UgjWQN_mqa8HvPJY")

    def test_get_ref(self):
        self.assertEqual(self.api.get_ref("San Francisco Grand opening").ref, "UgjWRd_mqbYHvPJa")

    def test_get_master(self):
        self.assertEqual(self.api.get_master().ref, "UgjWQN_mqa8HvPJY")
        self.assertEqual(self.api.get_master().id, "master")


class TestSearchFormTestCase(PrismicTestCase):
    def test_document(self):
        docs = [prismic.Document(doc) for doc in self.fixture_search]
        self.assertEqual(len(docs), 3)
        doc = docs[0]
        self.assertEqual(doc.slug, "vanilla-macaron")

    def test_empty_slug(self):
        doc_json = self.fixture_search[0]
        doc_json["slugs"] = None
        doc = prismic.Document(doc_json)
        self.assertEqual(doc.slug, "-")

    def test_as_html(self):
        doc_json = self.fixture_search[0]
        doc = prismic.Document(doc_json)
        expected_html = ("""<section data-field="product.allergens"><span class="text">Contains almonds, eggs, milk</span></section>"""
                         """<section data-field="product.image"><img src="https://wroomio.s3.amazonaws.com/micro/0417110ebf2dc34a3e8b7b28ee4e06ac82473b70.png" alt="" width="500" height="500" /></section>"""
                         """<section data-field="product.short_lede"><h2>Crispiness and softness, rolled into one</h2></section>"""
                         """<section data-field="product.testimonial_author[0]"><h3>Chef Guillaume Bort</h3></section><section data-field="product.related[0]"><a href="document/UdUjvt_mqVNObPeO">dark-chocolate-macaron</a></section>"""
                         """<section data-field="product.name"><h1>Vanilla Macaron</h1></section>"""
                         """<section data-field="product.related[1]"><a href="document/UdUjsN_mqT1ObPeM">salted-caramel-macaron</a></section>"""
                         """<section data-field="product.testimonial_quote[0]"><p>The taste of pure vanilla is very hard to tame, and therefore, most cooks resort to substitutes. <strong>It takes a high-skill chef to know how to get the best of tastes, and <strong><em></strong>Les Bonnes Choses<strong></em></strong>'s vanilla macaron does just that</strong>. The result is more than a success, it simply is a gastronomic piece of art.</p></section>"""
                         """<section data-field="product.flavour[0]"><span class="text">Vanilla</span></section>"""
                         """<section data-field="product.price"><span class="number">3.55</span></section><section data-field="product.color"><span class="color">#ffeacd</span></section>"""
                         """<section data-field="product.description"><p>Experience the ultimate vanilla experience. Our vanilla Macarons are made with our very own (in-house) <strong>pure extract of Madagascar vanilla</strong>, and subtly dusted with <strong>our own vanilla sugar</strong> (which we make from real vanilla beans).</p></section>""")
        doc_html = doc.as_html(lambda link_doc: "document/%s" % link_doc.id)
        # Comparing len rather than actual strings because json loading is not in a deterministic order for now
        self.assertEqual(len(expected_html), len(doc_html))

    def test_default_params_empty(self):
        form = self.api.form("everything")
        self.assertEqual(len(form.data), 0)

    def test_query_append_value(self):
        form = self.api.form("everything")
        form.query("[[bar]]")
        self.assertEqual(len(form.data), 1)
        self.assertEqual(form.data["q"], ["[[bar]]"])

    def test_ref_replace_value(self):
        form = self.api.form("everything")
        form.ref("foo")
        self.assertEqual(len(form.data), 1)
        self.assertEqual(form.data["ref"], "foo")
        form.ref("bar")
        self.assertEqual(len(form.data), 1)
        self.assertEqual(form.data["ref"], "bar")

    def test_set_page_size(self):
        form = self.api.form("everything")
        form.page_size(3)
        self.assertEqual(len(form.data), 1)
        self.assertEqual(form.data["pageSize"], 3)

    def test_set_page(self):
        form = self.api.form("everything")
        form.page(3)
        self.assertEqual(len(form.data), 1)
        self.assertEqual(form.data["page"], 3)


class TestFragmentsTestCase(PrismicTestCase):
    def setUp(self):
        super(TestFragmentsTestCase, self).setUp()
        doc_json = self.fixture_search[0]
        self.doc = prismic.Document(doc_json)

    def test_image(self):
        doc = self.doc
        self.assertEqual(doc.get_image("product.image", "main").width, 500)
        self.assertEqual(doc.get_image("product.image", "icon").width, 250)
        expected_html = \
            ("""<img """
             """src="https://wroomio.s3.amazonaws.com/micro/babdc3421037f9af77720d8f5dcf1b84c912c6ba.png" """
             """alt="" width="250" height="250" />""")
        self.assertEqual(expected_html, doc.get_image("product.image", "icon").as_html(PrismicTestCase.link_resolver))

    def test_number(self):
        doc = self.doc
        self.assertEqual(doc.get_number("product.price").__str__(), "3.55")

    def test_color(self):
        doc = self.doc
        self.assertEqual(doc.get_color("product.color").__str__(), "#ffeacd")

    def test_text(self):
        doc = self.doc
        self.assertEqual(doc.get_text("product.allergens").__str__(), "Contains almonds, eggs, milk")

        text = prismic.Fragment.Text("a&b 42 > 41")
        self.assertEqual(text.as_html, '<span class="text">a&amp;b 42 &gt; 41</span>', "HTML escape")

    def test_structured_text_heading(self):
        doc = self.doc
        html = doc.get_html("product.short_lede", lambda x: "/x")
        self.assertEqual("<h2>Crispiness and softness, rolled into one</h2>", html)

    def test_structured_text_paragraph(self):
        span_sample_data = {"type": "paragraph",
                            "text": "To be or not to be ?",
                            "spans": [
                                {"start": 3, "end": 5, "type": "strong"},
                                {"start": 16, "end": 18, "type": "strong"},
                                {"start": 3, "end": 5, "type": "em"}
                            ]}
        p = prismic.fragments.StructuredText([span_sample_data])
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, "<p>To <em><strong>be</strong></em> or not to <strong>be</strong> ?</p>")

        p = prismic.fragments.StructuredText([{"type": "paragraph", "text": "a&b 42 > 41", "spans": []}])
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, "<p>a&amp;b 42 &gt; 41</p>", "Paragraph HTML escape")

        p = prismic.fragments.StructuredText([{"type": "heading2", "text": "a&b 42 > 41", "spans": []}])
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, "<h2>a&amp;b 42 &gt; 41</h2>", "Header HTML escape")

    def test_spans(self):
        doc_json = self.fixture_spans_labels
        p = prismic.fragments.StructuredText(doc_json.get("value"))
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, ("""<p>Two <strong><em>spans</em> with</strong> the same start</p>"""
                                  """<p>Two <em><strong>spans</strong> with</em> the same start</p>"""
                                  """<p>Span till the <span class="tip">end</span></p>"""))

    def test_lists(self):
        doc_json = self.fixture_structured_lists[0]
        doc = prismic.Document(doc_json)
        doc_html = doc.get_structured_text("article.content").as_html(lambda x: "/x")
        expected = ("""<ul><li>Element1</li><li>Element2</li><li>Element3</li></ul>"""
                    """<p>Ordered list:</p><ol><li>Element1</li><li>Element2</li><li>Element3</li></ol>""")
        self.assertEqual(doc_html, expected)

    def test_empty_paragraph(self):
        doc_json = self.fixture_empty_paragraph
        doc = prismic.Document(doc_json)

        doc_html = doc.get_field('announcement.content').as_html(PrismicTestCase.link_resolver)
        expected = """<p>X</p><p></p><p>Y</p>"""
        self.assertEqual(doc_html, expected)

    def test_block_labels(self):
        doc_json = self.fixture_block_labels
        doc = prismic.Document(doc_json)

        doc_html = doc.get_field('announcement.content').as_html(PrismicTestCase.link_resolver)
        expected = """<p class="code">some code</p>"""
        self.assertEqual(doc_html, expected)

    def test_get_text(self):
        doc_json = self.fixture_search[0]
        doc = prismic.Document(doc_json)
        self.assertEqual(doc.get_text('product.description'), 'Experience the ultimate vanilla experience. Our vanilla Macarons are made with our very own (in-house) pure extract of Madagascar vanilla, and subtly dusted with our own vanilla sugar (which we make from real vanilla beans).')

    def test_document_link(self):
        test_paragraph = {
            "type": "paragraph",
            "text": "bye",
            "spans": [
                {
                    "start": 0,
                    "end": 3,
                    "type": "hyperlink",
                    "data": {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UbiYbN_mqXkBOgE2", "type": "article", "tags": ["form"], "slug": "-"
                            },
                            "isBroken": False
                        }
                    }
                },
                {"start": 0, "end": 3, "type": "strong"}
            ]
        }
        p = prismic.fragments.StructuredText([test_paragraph])

        def link_resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)

        p_html = p.as_html(link_resolver)
        self.assertEqual(p_html, """<p><strong><a href="/document/UbiYbN_mqXkBOgE2/-">bye</a></strong></p>""")

    def test_geo_point(self):
        store = prismic.Document(self.fixture_store_geopoint)
        geopoint = store.get_field("store.coordinates")
        self.assertEqual(geopoint.as_html,
                         ("""<div class="geopoint"><span class="latitude">37.777431</span>"""
                          """<span class="longitude">-122.415419</span></div>"""))

    def test_group(self):
        contributor = prismic.Document(self.fixture_groups)
        links = contributor.get_group("contributor.links")
        self.assertEquals(len(links.value), 2)

    def test_slicezone(self):
        self.maxDiff = 10000
        doc = prismic.Document(self.fixture_slices)
        slices = doc.get_slice_zone("article.blocks")
        slices_html = slices.as_html(PrismicTestCase.link_resolver)
        expected_html = (
            """<div data-slicetype="features" class="slice"><section data-field="illustration"><img src="https://wroomdev.s3.amazonaws.com/toto/db3775edb44f9818c54baa72bbfc8d3d6394b6ef_hsf_evilsquall.jpg" alt="" width="4285" height="709" /></section>"""
            """<section data-field="title"><span class="text">c'est un bloc features</span></section></div>\n"""
            """<div data-slicetype="text" class="slice"><p>C'est un bloc content</p></div>""")
        # Comparing len rather than actual strings because json loading is not in a deterministic order for now
        self.assertEqual(len(expected_html), len(slices_html))

    def test_composite_slices(self):
        self.maxDiff = 1000
        doc = prismic.Document(self.fixture_composite_slices)
        slices = doc.get_slice_zone("test.body")
        slices_html = slices.as_html(PrismicTestCase.link_resolver)
        expected_html = """<div data-slicetype="slice-a" class="slice"><section data-field="non-repeat-text"><p>Slice A non-repeat text</p></section><section data-field="non-repeat-title"><h1>Slice A non-repeat title</h1></section><section data-field="repeat-text"><p>Repeatable text A</p></section><section data-field="repeat-title"><h1>Repeatable title A</h1></section>
<section data-field="repeat-text"><p>Repeatable text B</p></section><section data-field="repeat-title"><h1>Repeatable title B</h1></section></div>
<div data-slicetype="slice-b" class="slice"><section data-field="image"><img src="https://prismic-io.s3.amazonaws.com/tails/014c1fe46e3ceaf04b7cc925b2ea7e8027dc607a_mobile_header_tp.png" alt="" width="800" height="500" /></section><section data-field="title"><h1>Slice A non-repeat title</h1></section></div>"""
        # Comparing len rather than actual strings because json loading is not in a deterministic order for now
        self.assertEqual(len(expected_html), len(slices_html))

    def test_image_links(self):
        self.maxDiff = 10000
        text = prismic.fragments.StructuredText(self.fixture_image_links.get('value'))

        self.assertEqual(
            text.as_html(PrismicTestCase.link_resolver),
            ("""<p>Here is some introductory text.</p>"""
             """<p>The following image is linked.</p>"""
             """<p class="block-img"><a href="http://google.com/">"""
             """<img src="http://fpoimg.com/129x260" alt="" width="260" height="129" /></a></p>"""
             """<p><strong>More important stuff</strong></p><p>The next is linked to a valid document:</p>"""
             """<p class="block-img"><a href="/document/UxCQFFFFFFFaaYAH/something-fantastic">"""
             """<img src="http://fpoimg.com/400x400" alt="" width="400" height="400" /></a></p>"""
             """<p>The next is linked to a broken document:</p><p class="block-img"><a href="#broken">"""
             """<img src="http://fpoimg.com/250x250" alt="" width="250" height="250" /></a></p>"""
             """<p>One more image, this one is not linked:</p><p class="block-img">"""
             """<img src="http://fpoimg.com/199x300" alt="" width="300" height="199" /></p>"""))

    def test_custom_html(self):
        self.maxDiff = 10000
        text = prismic.fragments.StructuredText(self.fixture_custom_html.get('value'))

        self.assertEqual(
            text.as_html(PrismicTestCase.link_resolver, PrismicTestCase.html_serializer),
            ("""<p>Here is some introductory text.</p>"""
             """<p>The following image is linked.</p>"""
             """<a href="http://google.com/"><img src="http://fpoimg.com/129x260" alt="" width="260" height="129" /></a>"""
             """<p><strong>More important stuff</strong></p><p>The next is linked to a valid document:</p>"""
             """<a href="/document/UxCQFFFFFFFaaYAH/something-fantastic">"""
             """<img src="http://fpoimg.com/400x400" alt="" width="400" height="400" /></a>"""
             """<p>The next is linked to a broken document:</p><a href="#broken">"""
             """<img src="http://fpoimg.com/250x250" alt="" width="250" height="250" /></a>"""
             """<p>One more image, this one is not linked:</p>"""
             """<img src="http://fpoimg.com/199x300" alt="" width="300" height="199" />"""
             """<p>This <a class="some-link" href="/document/UlfoxUnM0wkXYXbu/les-bonnes-chosess-internship-a-testimony">"""
             """paragraph</a> contains an hyperlink.</p>"""))


class PredicatesTestCase(PrismicTestCase):

    def test_at(self):
        f = self.api\
            .form("everything")\
            .ref(self.api.get_master())\
            .query(predicates.at('document.id', 'UlfoxUnM0wkXYXbZ'))
        self.assertEqual(f.data['q'], ["[[:d = at(document.id, \"UlfoxUnM0wkXYXbZ\")]]"])

    def test_not(self):
        f = self.api\
            .form("everything")\
            .ref(self.api.get_master())\
            .query(predicates.not_('document.id', 'UlfoxUnM0wkXYXbZ'))
        self.assertEqual(f.data['q'], ["[[:d = not(document.id, \"UlfoxUnM0wkXYXbZ\")]]"])

    def test_any(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.any('document.type', ['article', 'form-post']))
        self.assertEqual(f.data['q'], ['[[:d = any(document.type, ["article", "form-post"])]]'])

    def test_similar(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.similar('idOfSomeDocument', 10))
        self.assertEqual(f.data['q'], ['[[:d = similar("idOfSomeDocument", 10)]]'])

    def test_multiple_predicates(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(
                predicates.month_after('my.form-post.publication-date', 4),
                predicates.month_before('my.form-post.publication-date', 'December')
            )
        self.assertEqual(f.data['q'], ['[[:d = date.month-after(my.form-post.publication-date, 4)][:d = date.month-before(my.form-post.publication-date, "December")]]'])

    def test_number_lt(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.lt('my.form-post.publication-date', 4))
        self.assertEqual(f.data['q'], ['[[:d = number.lt(my.form-post.publication-date, 4)]]'])

    def test_number_in_range(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.in_range('my.product.price', 2, 4.5))
        self.assertEqual(f.data['q'], ['[[:d = number.inRange(my.product.price, 2, 4.5)]]'])

    def test_geopoint_near(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.near('my.store.coordinates', 40.689757, -74.0451453, 15))
        self.assertEqual(f.data['q'], ['[[:d = geopoint.near(my.store.coordinates, 40.689757, -74.0451453, 15)]]'])


class TestCache(unittest.TestCase):

    def setUp(self):
        self.cache = ShelveCache("cachetest")

    def test_set_get(self):
        self.cache.set("foo", "bar", 3600)
        self.assertEqual(self.cache.get("foo"), "bar")

    def test_expiration(self):
        self.cache.set("toto", "tata", 2)
        time.sleep(3)
        self.assertIsNone(self.cache.get("toto"))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for Prismic library"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

from prismic.cache import ShelveCache, NoCache
from prismic.exceptions import InvalidTokenError, AuthorizationNeededError, \
    UnexpectedError
from .test_prismic_fixtures import fixture_api, fixture_search, fixture_groups, \
    fixture_structured_lists, fixture_empty_paragraph, fixture_store_geopoint, \
    fixture_image_links, fixture_spans_labels, fixture_block_labels, fixture_custom_html
import json
import logging
import datetime
import prismic
from prismic import predicates
import unittest

# logging.basicConfig(level=logging.DEBUG)
# log = logging.getLogger(__name__)


class DocTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        """Teardown."""

    def test_api(self):
        api = prismic.get("https://micro.prismic.io/api")
        self.assertIsNotNone(api)

    def test_form(self):
        api = prismic.get("https://micro.prismic.io/api")
        response = api.form("everything").ref(api.get_master())\
            .query(predicates.at("document.type", "all"))\
            .submit()
        self.assertGreaterEqual(response.results_size, 2)

    def test_api_private(self):
        try:
            # This will fail because the token is invalid, but this is how to access a private API
            api = prismic.get('https://micro.prismic.io/api', 'MC5-XXXXXXX-vRfvv70')
            self.fail('Should have thrown')
        except InvalidTokenError as e:
            pass

    def test_references(self):
        preview_token = 'MC5VcXBHWHdFQUFONDZrbWp4.77-9cDx6C3lgJu-_vXZafO-_vXPvv73vv73vv70777-9Ju-_ve-_vSLvv73vv73vv73vv70O77-977-9Me-_vQ'
        api = prismic.get('https://micro.prismic.io/api', preview_token)
        release_ref = api.get_ref('myrelease')
        response = api.query(predicates.at("document.type", "all"), ref=release_ref)
        self.assertGreaterEqual(response.results_size, 1)

    def test_orderings(self):
        api = prismic.get('https://micro.prismic.io/api')
        response = api.query(predicates.at("document.type", "all"), page_size=2, orderings='[my.all.number desc]')
        # The documents are now ordered using the 'number' field, highest first
        docs = response.documents
        self.assertGreaterEqual(docs[0].get_number('all.number').value, docs[1].get_number('all.number').value)

    def test_as_html(self):
        api = prismic.get("http://micro.prismic.io/api")
        doc = api.get_by_uid('all', 'all')
        def link_resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)
        html = doc.as_html(link_resolver)
        self.assertIsNotNone(html)

    def test_html_serializer(self):
        api = prismic.get("http://micro.prismic.io/api")
        doc = api.get_by_uid('all', 'all')

        def link_resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)

        def html_serializer(element, content):
            if isinstance(element, prismic.fragments.Block.Image):
                # Don't wrap images in a <p> tag
                return element.get_view().as_html(link_resolver)
            if isinstance(element, prismic.fragments.Span.Hyperlink):
                # Add a class to links
                return """<a class="some-link" href="%s">""" % element.get_url(link_resolver) + content + "</a>"
            return None

        html = doc.get_structured_text("all.stext").as_html(link_resolver, html_serializer)
        self.assertIsNotNone(html)

    def test_get_text(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')
        author = doc.get_text("all.text")
        self.assertEqual(author, "all")

    def test_get_number(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')
        price = doc.get_number("all.number").value
        self.assertEqual(price, 20.0)

    def test_get_range(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')
        price = doc.get_range("all.range").value
        self.assertEqual(price, '38')

    def test_images(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')
        url = doc.get_image('all.image').url
        self.assertEqual(
            url,
            'https://micro.cdn.prismic.io/micro/e185bb021862c2c03a96bea92e170830908c39a3_thermometer.png')

    def test_date(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')
        date = doc.get_date("all.date")
        self.assertEqual(date.as_datetime, datetime.datetime(2017, 1, 16, 0, 0))

    def test_date_html(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')

        date = doc.get_date("all.date")
        self.assertEqual(date.as_html, '<time>2017-01-16</time>')

    def test_timestamp(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')

        timestamp = doc.get_timestamp("all.timestamp")
        self.assertEqual(timestamp.as_datetime, datetime.datetime(2017, 1, 16, 7, 25, 35))

    def test_timestamp_html(self):
        api = prismic.get('https://micro.prismic.io/api')
        doc = api.get_by_uid('all', 'all')

        timestamp = doc.get_timestamp("all.timestamp")
        self.assertEqual(timestamp.as_html, '<time>2017-01-16T07:25:35+0000</time>')

    def test_group(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"documents\":{\"type\":\"Group\",\"value\":[{\"linktodoc\":{\"type\":\"Link.document\",\"value\":{\"document\":{\"id\":\"UrDejAEAAFwMyrW9\",\"type\":\"doc\",\"tags\":[],\"slug\":\"installing-meta-micro\"},\"isBroken\":false}},\"desc\":{\"type\":\"StructuredText\",\"value\":[{\"type\":\"paragraph\",\"text\":\"A detailed step by step point of view on how installing happens.\",\"spans\":[]}]}},{\"linktodoc\":{\"type\":\"Link.document\",\"value\":{\"document\":{\"id\":\"UrDmKgEAALwMyrXA\",\"type\":\"doc\",\"tags\":[],\"slug\":\"using-meta-micro\"},\"isBroken\":false}}}]}}}}"
        document = prismic.Document(json.loads(data))
        def resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)
        group = document.get_group("article.documents")
        docs = (group and group.value) or []
        for doc in docs:
            desc = doc.get_structured_text("desc")
            link = doc.get_link("linktodoc")
        self.assertEqual(docs[0].get_structured_text("desc").as_html(resolver), "<p>A detailed step by step point of view on how installing happens.</p>")

    def test_link(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"source\":{\"type\":\"Link.document\",\"value\":{\"document\":{\"id\":\"UlfoxUnM0wkXYXbE\",\"type\":\"product\",\"tags\":[\"Macaron\"],\"slug\":\"dark-chocolate-macaron\"},\"isBroken\":false}}}}}"
        document = prismic.Document(json.loads(data))
        def resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)
        source = document.get_link("article.source")
        url = source and source.get_url(resolver)
        self.assertEqual(url, "/document/UlfoxUnM0wkXYXbE/dark-chocolate-macaron")

    def test_embed(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"video\":{\"type\":\"Embed\",\"value\":{\"oembed\":{\"provider_url\":\"http://www.youtube.com/\",\"type\":\"video\",\"thumbnail_height\":360,\"height\":270,\"thumbnail_url\":\"http://i1.ytimg.com/vi/baGfM6dBzs8/hqdefault.jpg\",\"width\":480,\"provider_name\":\"YouTube\",\"html\":\"<iframe width=\\\"480\\\" height=\\\"270\\\" src=\\\"http://www.youtube.com/embed/baGfM6dBzs8?feature=oembed\\\" frameborder=\\\"0\\\" allowfullscreen></iframe>\",\"author_name\":\"Siobhan Wilson\",\"version\":\"1.0\",\"author_url\":\"http://www.youtube.com/user/siobhanwilsonsongs\",\"thumbnail_width\":480,\"title\":\"Siobhan Wilson - All Dressed Up\",\"embed_url\":\"https://www.youtube.com/watch?v=baGfM6dBzs8\"}}}}}}"
        document = prismic.Document(json.loads(data))
        video = document.get_embed("article.video")
        # Html is the code to include to embed the object, and depends on the embedded service
        html = video and video.as_html
        self.assertEqual(html, "<div data-oembed=\"https://www.youtube.com/watch?v=baGfM6dBzs8\" data-oembed-type=\"video\" data-oembed-provider=\"YouTube\"><iframe width=\"480\" height=\"270\" src=\"http://www.youtube.com/embed/baGfM6dBzs8?feature=oembed\" frameborder=\"0\" allowfullscreen></iframe></div>")

    def test_color(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"background\":{\"type\":\"Color\",\"value\":\"#000000\"}}}}"
        document = prismic.Document(json.loads(data))
        bgcolor = document.get_color("article.background")
        hex = bgcolor.value
        self.assertEqual(hex, "#000000")

    def test_geopoint(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"location\":{\"type\":\"GeoPoint\",\"value\":{\"latitude\":48.877108,\"longitude\":2.333879}}}}}"
        document = prismic.Document(json.loads(data))
        # "near" predicate for GeoPoint fragments
        near = predicates.near("my.store.location", 48.8768767, 2.3338802, 10)

        # Accessing GeoPoint fragments
        place = document.get_geopoint("article.location")
        coordinates = place and ("%.6f,%.6f" % (place.latitude, place.longitude))
        self.assertEqual(coordinates, "48.877108,2.333879")

    def test_cache(self):
        # Just implement your own cache object by duck-typing
        # https://github.com/prismicio/python-kit/blob/master/prismic/cache.py
        no_cache = NoCache()
        # This api will use the custom cache object
        api = prismic.get('https://micro.prismic.io/api', cache=no_cache)
        self.assertIsNotNone(api)


if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-

"""
prismic.api
~~~~~~~~~~~

This module implements the Prismic API.

"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

import sys
from copy import copy, deepcopy
from .connection import get_json, urlparse
from .experiments import Experiments
from . import predicates
from .exceptions import RefMissing
from .fragments import Fragment
from collections import OrderedDict

from .utils import string_types
import logging

log = logging.getLogger(__name__)


def get(url, access_token=None, cache=None, request_handler=None):
    """Fetches the prismic api JSON.
    Returns :class:`Api <Api>` object.

    :param url: URL to the api of the repository (mandatory).
    :param access_token: The access token (optional).
    :param cache: The cache object. Optional, will default to a file-based cache if None is passed.
    :param request_handler: The request handler. Optional, will default to a request handler based on requests module.
    """
    return Api(
        get_json(url, access_token=access_token, cache=cache, ttl=5, request_handler=request_handler),
        access_token,
        cache,
        request_handler
    )


class Api(object):
    """
    A Prismic API, pointing to a specific repository. Use prismic.api.get() to fetch one.

    :ivar dict bookmarks: all bookmarks, as a dict from name to document id
    :ivar array<str> types: all available types
    :ivar array<str> tags: all available tags
    :ivar Experiments experiments: information about current experiments
    :ivar str access_token: current access token (may be None)
    """

    def __init__(self, data, access_token, cache, request_handler):
        self.cache = cache
        self.request_handler = request_handler
        self.refs = [Ref(ref) for ref in data.get("refs")]
        self.bookmarks = data.get("bookmarks")
        self.types = data.get("types")
        self.tags = data.get("tags")
        self.forms = data.get("forms")
        for name in self.forms:
            fields = self.forms[name].get("fields")
            for field in fields:
                if field == "q":
                    fields[field].update({"multiple": True})
        self.experiments = Experiments.parse(data.get("experiments"))
        self.oauth_initiate = data.get("oauth_initiate")
        self.oauth_token = data.get("oauth_token")
        self.access_token = access_token

        self.master = ([ref for ref in self.refs if ref.is_master_ref][:1] or [None])[0]
        if not self.master:
            log.error("No master reference found")

    def preview_session(self, token, link_resolver, default_url):
        """Return the URL to display a given preview

        :param token as received from Prismic server to identify the content to preview
        :param link_resolver the link resolver to build URL for your site
        :param default_url the URL to default to return if the preview doesn't correspond to a document
                       (usually the home page of your site)

        :return: the URL to redirect the user to
        """
        main_document_id = get_json(token, request_handler=self.request_handler).get("mainDocument")
        if main_document_id is None:
            return default_url
        doc = self.get_by_id(main_document_id, ref=token)
        if doc is None == 0:
            return default_url
        return link_resolver(doc.as_link())

    def get_ref(self, label):
        """Get the :class:`Ref <Ref>` with a specific label.
        Returns :class:`Ref <Ref>` object.

        :param label: Name of the label.
        """
        ref = [ref for ref in self.refs if ref.label == label]
        return ref[0] if ref else None

    def get_master(self):
        """Returns current master :class:`Ref <Ref>` object."""
        return self.master

    def form(self, name):
        """Constructs the form with data from Api.
        Returns :class:`SearchForm <SearchForm>` object.

        :param name: Name of the form.
        """
        form = self.forms.get(name)
        if form is None:
            raise Exception("Bad form name %s, valid form names are: %s" % (name, ', '.join(self.forms)))
        return SearchForm(self.forms.get(name), self.access_token, self.cache, self.request_handler)

    def query(self, q, ref=None, page_size=None, page=None, orderings=None, after=None, fetch_links=None):
        if ref is None:
            ref = self.get_master()
        form = self.form('everything').ref(ref)
        if page_size is not None:
            form.page_size(page_size)
        if page is not None:
            form.page(page)
        if orderings is not None:
            form.orderings(orderings)
        if after is not None:
            form.after(after)
        if fetch_links is not None:
            form.fetch_links(fetch_links)
        return form.query(q).submit()

    def query_first(self, q, ref=None):
        documents = self.query(q, ref, page_size=1, page=1).documents
        if len(documents) > 0:
            return documents[0]

    def get_by_uid(self, type, uid, ref=None):
        return self.query_first(predicates.at('my.' + type + '.uid', uid), ref)

    def get_by_id(self, id, ref=None):
        return self.query_first(predicates.at('document.id', id), ref)

    def get_by_ids(self, ids, ref=None, page_size=None, page=None, orderings=None, after=None, fetch_links=None):
        return self.query(
            predicates.in_('document.id', ids),
            ref,
            page_size=page_size,
            page=page,
            orderings=orderings,
            after=after,
            fetch_links=fetch_links
        )

    def get_single(self, type, ref=None):
        return self.query_first(predicates.at('document.type', type), ref)

class Ref(object):
    """
    A Prismic.io Reference (corresponds to a release)
    """

    def __init__(self, data):
        self.id = data.get("id")
        self.ref = data.get("ref")
        self.label = data.get("label")
        self.is_master_ref = data.get("isMasterRef")
        self.scheduled_at = data.get("scheduledAt")


class SearchForm(object):
    """Form to search for documents. Most of the methods return self object to allow chaining.
    """

    def __init__(self, form, access_token, cache, request_handler):
        self.action = form.get("action")
        self.method = form.get("method")
        self.enctype = form.get("enctype")
        self.fields = form.get("fields") or {}
        self.data = {}
        # default values
        for field, value in list(self.fields.items()):
            if value.get("default"):
                self.set(field, value["default"])
        self.access_token = access_token
        self.cache = cache
        self.request_handler = request_handler

    def ref(self, ref):
        """:param ref: A :class:`Ref <Ref>` object or an string."""

        if isinstance(ref, Ref):
            ref = ref.ref

        return self.set('ref', ref)

    @staticmethod
    def _serialize(field):
        if isinstance(field, string_types):
            if field.startswith('my.') or field.startswith('document.') or field == 'document':
                return field
            else:
                return '"' + field + '"'
        elif hasattr(field, '__iter__'):
            strings = []
            for item in field:
                strings.append(SearchForm._serialize(item))
            return "[" + ", ".join(strings) + "]"
        else:
            return str(field)

    def query(self, *argv):
        """:param argv: Either a string query, or any number of Array corresponding to predicates.

        See the :mod:`prismic.predicates <prismic.predicate>` module for helper functions.
        """
        if len(argv) == 0:
            return self
        if isinstance(argv[0], string_types):
            q = argv[0]
        else:
            q = "["
            for predicate in argv:
                op = predicate[0]
                args = []
                for arg in predicate[1:]:
                    args.append(SearchForm._serialize(arg))
                q += "[:d = %(op)s(%(args)s)]" % {
                    'op': op,
                    'args': ", ".join(args)
                }
            q += "]"
        return self.set('q', q)

    def set(self, field, value):
        form_field = self.fields.get(field)
        if form_field and form_field.get("multiple"):
            if not self.data.get(field):
                self.data.update({field: []})
            self.data[field].append(value)
        else:
            self.data.update({field: value})
        return self

    def orderings(self, orderings):
        """Sets the query orderings

        :param orderings String with the orderings predicate
        :returns: the SearchForm instance to chain calls
        """
        return self.set("orderings", orderings)

    def submit_assert_preconditions(self):
        if self.data.get('ref') is None:
            raise RefMissing()

    def submit(self):
        """
        Submit the query to the Prismic.io server

        :return: :class:`Response <prismic.api.Response>`
        """
        self.submit_assert_preconditions()
        return Response(get_json(
            self.action,
            self.data,
            self.access_token,
            self.cache,
            request_handler=self.request_handler
        ))

    def page(self, page_number):
        """Set query page number

        :param page_number: int representing the page number
        """
        return self.set("page", page_number)

    def page_size(self, nb_results):
        """Set query page size

        :param nb_results: int representing the number of results per page
        """
        return self.set("pageSize", nb_results)

    def after(self, doc_id):
        """Start the result set after the given id

        :param doc_id: id of the reference document
        """
        return self.set("after", doc_id)

    def fetch(self, fields):
        """ Restrict the results document to the specified fields

        :param fields: The list of fields, array or comma separated string
        """
        if isinstance(fields, list):
            fields = ",".join(fields)
        return self.set("fetch", fields)

    def fetch_links(self, fields):
        """ Include the requested fields in the DocumentLink instances in the result

        :param fields: The list of fields, array or comma separated string
        """
        if isinstance(fields, list):
            fields = ",".join(fields)
        return self.set("fetchLinks", fields)

    def pageSize(self, nb_results):
        """Deprecated: use page_size instead
        """
        return self.page_size(nb_results)

    def count(self):
        """Count the total number of results
        """
        return copy(self).pageSize(1).submit().total_results_size

    def __copy__(self):
        cp = type(self)({}, self.access_token, self.cache, self.request_handler)
        cp.action = deepcopy(self.action)
        cp.method = deepcopy(self.method)
        cp.enctype = deepcopy(self.enctype)
        cp.fields = deepcopy(self.fields)
        cp.data = deepcopy(self.data)
        return cp


class Response(object):
    """
    Prismic's response to a query.

    :ivar array<prismic.api.Document> documents: the documents of the current page
    :ivar int page: the page in this result, starting by 1
    :ivar int results_per_page: max result in a page
    :ivar int total_results_size: total number of results for this query
    :ivar int total_pages: total number of pages for this query
    :ivar str next_page: URL of the next page (may be None if on the last page )
    :ivar str prev_page: URL of the previous page (may be None)
    :ivar int results_size: number of results actually returned for the current page
    """

    def __init__(self, data):
        self._data = data
        self.documents = [Document(d) for d in data.get("results")]
        self.page = data.get("page")
        self.next_page = data.get("next_page")
        self.prev_page = data.get("prev_page")
        self.results_per_page = data.get("results_per_page")
        self.total_pages = data.get("total_pages")
        self.total_results_size = data.get("total_results_size")
        self.results_size = data.get("results_size")

    def __getattr__(self, name):
        return self._data.get(name)

    def __repr__(self):
        return "Response %s" % self._data


class Document(Fragment.WithFragments):
    """
    Represents a Prismic.io Document

    :ivar str id: document id
    :ivar str uid: document uid
    :ivar str type:
    :ivar str href:
    :ivar array<str> tags:
    :ivar array<str> slugs:
    """

    def __init__(self, data):
        Fragment.WithFragments.__init__(self, OrderedDict())
        self._data = data

        fragments = {}
        if "data" in data:
            fragments = data.get("data").get(self.type)
        for (fragment_name, fragment_value) in list(fragments.items()):
            f_key = "%s.%s" % (self.type, fragment_name)

            if isinstance(fragment_value, list):
                for index, fragment_value_element in enumerate(fragment_value):
                    self.fragments["%s[%s]" % (f_key, index)] = Fragment.from_json(
                        fragment_value_element)

            elif isinstance(fragment_value, dict):
                self.fragments[f_key] = Fragment.from_json(fragment_value)

        self.slugs = ["-"]
        if data.get("slugs") is not None:
            self.slugs = [Document.__unquote(slug) for slug in data.get("slugs")]

    @staticmethod
    def __unquote(s):
        if sys.version_info >= (3, 0):
            return urlparse.unquote(s)
        else:
            return urlparse.unquote(s.encode('utf8')).decode('utf8')

    def as_link(self):
        """
        Convert the current document to a DocumentLink

        :return: :class:`DocumentLink <prismic.api.Fragment.DocumentLink>`
        """
        data = self._data.copy()
        data['slug'] = self.slug
        return Fragment.DocumentLink({
            'document': data
        })

    def __getattr__(self, name):
        if name.startswith('__'):
            raise AttributeError
        return self._data.get(name, None)

    @property
    def slug(self):
        """
        Return the most recent slug

        :return: str slug
        """
        return self.slugs[0] if self.slugs else "-"

    def __repr__(self):
        return "Document %s" % self.fragments

